/*****************************************************************************
 *
 * nst_X11.h  -  wrappers for nst graphics routines using X11
 *
 * Copyright Helge Ritter, 1994
 *
 * $Source: /vol/ni/src/cvsroot/nst5/nstsrc/nst_X11.h,v $
 *
 * $Log: nst_X11.h,v $
 * Revision 1.41  2005/12/16 08:01:37  rhaschke
 * using recursive X11EventQMutex simplifies some things and prohibits deadlocks
 *
 * Revision 1.40  2005/11/07 07:52:24  rhaschke
 * added nst_locate() declaration
 *
 * Revision 1.39  2005/11/04 08:30:49  rhaschke
 * cleanup of shortcuts winget -> nst_winget etc.
 *
 * Revision 1.38  2005/10/21 15:07:21  rhaschke
 * cleanup
 *
 * Revision 1.37  2005/10/17 07:58:26  rhaschke
 * old open_window[0|1] now merged in create_window
 * routines to set resize hints for the window manager
 * Redraw method renamed to ResizeNotify, which better matches purpose
 * clean computation of borderwidth
 *
 * Revision 1.36  2005/10/14 12:06:57  rhaschke
 * removed nst_satisfy_redraw_requests()
 *
 * Revision 1.35  2005/10/14 11:28:23  rhaschke
 * cleanup
 *
 * Revision 1.34  2005/09/29 10:38:02  rhaschke
 * reenabled service_idle_tasks during NST event processing
 * several minor bug fixes
 *
 * Revision 1.33  2005/09/18 21:28:50  rhaschke
 * improved event handling
 *
 * Revision 1.32  2005/09/02 13:57:33  rhaschke
 * cleanup in declarations
 * nst_[scrn,X11,prog]: adjustments and additions for NST7's neosrc
 *
 * Revision 1.31  2005/07/12 07:50:18  rhaschke
 * added declarations
 *
 * Revision 1.30  2003/02/07 10:23:02  jontrup
 * (new feature): Added function nst_SetXRootWindow(). When using the Nstloader
 * mechanism this can be used to set the root window to the top level XWindow of
 * an embedding application. In this way a Neo circuit might be embedded within
 * other applications.
 *
 * Revision 1.29  2002/08/15 09:31:48  jontrup
 * Added nst_raise_window() function to raise a window.
 *
 * Revision 1.28  2002/08/12 07:01:00  rhaschke
 * added some declarations of exported functions
 *
 * Revision 1.26  2002/03/22 11:37:05  rhaschke
 * added some function declarations / includes to avoid compiler warnings
 *
 * Revision 1.25  2002/03/15 15:23:31  helge
 * some changes required to use nst_XRGBrectwrite() for writing RGB triples.
 * nst_handle_expose_event() now only called for expose events with count=0
 * (count>0 events are redundant and their redraws should be suppressed).
 * Also bracketed handler body with winget()/winset() to guarantee
 * preservation of window context.
 *
 * Revision 1.24  2002/02/07 13:00:58  jontrup
 * (minor change) added some declarations to header files.
 *
 * Revision 1.23  2002/01/15 09:13:22  helge
 * - added definitions so that each font has an "emphasized" partner font
 * - replaced an erroneous nst_ps_circ() by nst_ps_arc()
 *
 * Revision 1.22  2001/08/03 13:57:07  jontrup
 * Included a slightly modified version of T. Twellmanns grab_wdw
 * unit into the pixel folder. It allows to grab named NST windows
 * and to write their contents as ppm or gif files.
 *
 * Revision 1.21  2001/07/02 10:51:35  helge
 * (new feature) now each window can have a separate background and
 * foreground color (BLACK/WHITE is default for plot windows).
 * Former NST color index 1 is no longer white, but foreground
 * (and will thus follow background choice). For constant white
 * use color index 8, for constant black 9.
 *
 * Revision 1.20  2000/11/06 11:56:02  helge
 * new routine nst_peek_event(void)
 *
 * Revision 1.19  2000/06/24 19:31:08  helge
 * - improved font handling to allow NST units load user-specified fonts.
 * - better tracking of window enter/leave events
 *
 * Revision 1.18  2000/02/01 15:20:43  helge
 * - added nst_function_key() to evaluate function keys.
 * - some changes to internal handling of window resize and redraw requests
 *   (required for nstplugin and new zoom buttons after recent changes)
 *
 * Revision 1.17  2000/01/26 11:38:51  helge
 * some bugs removed. Added new button type: %z and %Z buttons to create
 * "folds" in GUIs for better structuring (documentation to follow)
 *
 * Revision 1.16  2000/01/17 10:54:02  helge
 * new routines nst_XGetKeyPair(), nst_return_key(), nst_cancel_key() and
 *              nst_alt_cmd() to support accelerator keys for Neo/NST
 *
 * Revision 1.15  2000/01/10 18:11:53  helge
 * new function: nst_wait_for_widget_event()
 * better font loading
 *
 * Revision 1.14  1999/09/27 14:55:40  helge
 * some additions for use by new nstplugin
 *
 * Revision 1.13  1999/05/31 18:14:58  helge
 * Added inob-field (inob=interface-object) in *unitptr struct to allow some
 * Neo-related extensions. Other changes mainly removal of files from standard
 * libnst.a into dynamic folders (moved into foldersrc, cf. files added there)
 *
 * Revision 1.11  1999/02/14 17:21:33  helge
 * some auxiliary routines (nst_is_in_window() etc) to support neo's new
 * autostart-feature. nst_XIfEvent() now returns int.
 *
 * Revision 1.10  1997/07/14 08:05:15  jan
 * Minor additions to match nst_X11.c
 *
 * Revision 1.9  1997/07/11 11:13:04  helge
 * some bugs removed, some things improved. See file "log" for details.
 *
 * Revision 1.7  1997/05/02 09:30:47  helge
 * removed nst_aux2.h from these headers
 *
 * Revision 1.6  1997/03/25 12:11:30  jan
 * Added missing declaration.
 *
 * Revision 1.5  1997/03/21 12:46:04  jan
 * Declaration fixes.
 *
 * Revision 1.4  1997/03/18 13:51:08  jan
 * Added missing 'extern' declarations.
 *
 * Revision 1.3  1997/03/14 10:29:53  jan
 * Cleaner emergency definition of constants belonging to 'gl/get.h'.
 *
 * Revision 1.2  1997/03/13 10:48:53  jan
 * Dependency cleanup.
 *
 *
 *****************************************************************************/

#ifndef __NST_X11_H
#define __NST_X11_H

#include <X11/Xlib.h>
#include <X11/keysym.h>
#ifdef irix5
#include <gl/get.h>
#else
#define DMRGB       0
#define DMDOUBLE    2
#define DMRGBDOUBLE 5
#define BCKBUFFER   0x1
#define FRNTBUFFER  0x2
#endif
#include "nst_aux2.h"

#define EM_BASE  5   /* offset between normal an emphasized font */
#define NST_ANY_WINDOW    0
#define NST_CMAP_WINDOW   1
#define NST_RGB_WINDOW    2
#define NST_RGB_3D_WINDOW 3
 
extern int giNstScreenDepth;
extern int giNstWidgetFont;
extern float gfNstCurrentFontDx;
extern float gfNstCurrentFontDy;
extern unsigned int giNstWdwBorderwidth;
extern int          giNstWdwTitlebarwidth;

extern int nst_init_X11(void);
extern int nst_to_gl_color(int);
extern int nst_cmap_to_red(int);
extern int nst_cmap_to_green(int);
extern int nst_cmap_to_blue(int);
extern void (*nst_XFlush)(void);
extern void nst_XSync(void);
extern int nst_XPaste(textwinstate *t);
extern int nst_XBlackpixel(void);
extern unsigned long nst_XGetKey(void);
extern unsigned long nst_XQueryKeys(int,int*);
extern void nst_wait_until_no_key_pressed(void);
extern int nst_query_pointer(Window*, int *piX,int *piY, int *piButtons);
extern void nst_query_buttons(int *piButtons);
extern void nst_pointer_xy(int*,int*,int*);
extern int nst_window_entered(void);
extern int nst_locate(int *piX, int *piY, char *pc, int* piButtons, int iMask);
extern int nst_wait_for_event(XEvent*);
extern int nst_get_window_or_subwindow_event(int,XEvent*,int bWait);
extern int nst_wait_for_window_or_subwindow_event(int,XEvent*);
extern int nst_wait_for_widget_event(int);
extern int nst_peek_event(void);
extern void nst_put_back_event(XEvent*);
extern void nst_remove_events1_except(long int, int, int*);
extern void nst_remove_events(long int,int);
extern void nst_set_expose_event_handler(int (*expose_event_handler)(int));
extern void nst_process_expose_events(void);
extern int  nst_evaluate_keypress_event(XEvent*, char *pcCh);
extern int  nst_evaluate_pointer_event(XEvent*, int *piX, int *piY, int *piButtons);
extern int nst_is_in_window(int);
extern int nst_is_in_subwindow(int);
extern int nst_window_size(int*, int*, int);
extern Window nst_XWindow_of(int);
extern int nst_window_from_XWindow(Window iXWdw);
void nst_unregister_window_tree(Display*,Window, void (*)(Display*,Drawable));
extern int nst_connect_psfile_to_wdw(FILE*,int);
extern void nst_disconnect_psfile(void);
extern int nst_open_subwindow(int,int,int,int,int,int);
extern int nst_create_subwindow(int,char*,int,int,int,int,int,int);

/* wrappers for nst_create_window (with specific type): */
extern int nst_open_window(int iXll,int iYll,int iDx,int iDy,char *pcTitle);
extern int nst_open_rgb_window(int iXll,int iYll,int iDx,int iDy,char *pcTitle);
extern int nst_open_rgb_3d_window(int iXll,int iYll,int iDx,int iDy,char *pcTitle);

/* these use a predefined window id (from previous open) */
extern int nst_reopen_window(int iId, int iXll,int iYll,int iDx,int iDy,char *pcTitle);
extern int nst_reopen_rgb_window(int iId, int iXll,int iYll,int iDx,int iDy,char *pcTitle);
extern int nst_reopen_rgb_3d_window(int iId, int iXll,int iYll,int iDx,int iDy,char *pcTitle);

/* handling window manager size hints */
extern void nst_activate_constraints ();
extern void nst_constrain_minsize (int iDx, int iDy);
extern void nst_constrain_maxsize (int iDx, int iDy);
extern void nst_constrain_aspect (int iX, int iY);
extern void nst_constrain_resize (int iDx, int iDy);

extern void nst_set_exit_request(void);
extern void nst_clear_exit_request(void);
extern void nst_set_window_create_hook(void *);
extern void nst_set_window_delete_hook(void *);
extern int  nst_set_window_font(int,char*);
extern int  nst_XWindow_belongs_to_circuit(Window,unitptr);
extern int  nst_locate_requests_exit(void);
extern void* nst_save_graphics_state(void*);
extern void  nst_restore_graphics_state(void*);
extern void nst_use_frontbuffer(int,int);
extern int  nst_use_backbuffer(int,int);
extern int nst_window_type(int);
extern int nst_is_rgb_3d_window(void);
extern int nst_set_cursor(int,int);
extern void nst_map_window(int);
extern void nst_unmap_window(int);
extern void nst_raise_window(int);
extern int nst_set_resize_notify_method(int,void (*)(int));
extern int nst_set_event_handler(int,void (*)(XEvent*, void*), void*);

extern void nst_offscreen_windows(int iEnable);
extern void nst_restrict_focus_to_main_window(void);

extern int nst_reserve_window_id(void);

extern int nst_alt_cmd(void);
extern int nst_return_key(void);
extern int nst_cancel_key(void);
extern int nst_function_key(void);

extern Window  *nst_XWindow(void);
extern Window   nst_replace_windows(Display*,Window,Window);
extern Window   nst_reparent_windows(Display*,Window,int,int,Window);
extern Display *nst_XDisplay(void);
extern Visual  *nst_XCMapVisual(void);
extern Visual  *nst_XRGBVisual(void);
extern int      nst_XCMap_depth(void);
extern int      nst_XRGB_depth(void);
extern Colormap nst_XCMap_colormap(void);
extern Colormap nst_XRGB_colormap(void);

extern int  nst_winget(void);
extern void nst_winset(int);
extern void nst_color(int);
extern void nst_winclose(int);
extern int  nst_winremove(int);
extern void nst_winmove(int,int);
extern void nst_subwinmove(int, int, int, int);
extern void nst_winresize(int,int,int);
extern void nst_warp_pointer(int,int);
extern int nst_getlwidth(void);
extern int nst_getplanes(void);
extern int nst_getdisplaymode(void);
extern void nst_swap_drawables_of(int,int);
extern void nst_linewidth(int);
extern void nst_set_foreground(int);
extern void nst_set_background(int);
extern void nst_set_window_ground(int,int,int);
extern void nst_get_foreground(int*);
extern void nst_get_background(int*);
extern void nst_RGBcolor(int,int,int);
extern void nst_dashed(int);
extern void nst_inverse(int);
extern void nst_clear(void);
extern void nst_ortho2(float,float,float,float);
extern void nst_winpop(void);
extern void nst_winpush(void);
extern void nst_getorigin(int*,int*);
extern void nst_getsize(int*,int*);
extern void nst_getsize_fast(int*,int*);
extern int  nst_getcolor(void);
extern int  nst_colorname_to_id(char*);
extern void nst_wintitle(char*);
extern void nst_move2i(int,int);
extern void nst_cmov(int,int);
extern void nst_cmov2(float,float);
extern void nst_cmov2i(int,int);
extern void nst_draw2i(int,int);
extern void nst_rdr2i(int,int);
extern void nst_rmv2i(int,int);
extern void nst_draw2(float,float);
extern void nst_rdr2(float,float);
extern void nst_rmv2(float,float);
extern void nst_move2(float,float);
extern void nst_circ(float,float,float);
extern void nst_pnt0(float,float);
extern void nst_pnt2(float,float);
extern void nst_rect(float,float,float,float);
extern void nst_rectf(float,float,float,float);
extern void nst_recti(int,int,int,int);
extern void nst_arrow(int,int,int,int,int,int,int,int);
extern void nst_rectfi(int,int,int,int);
extern void nst_arrowfi(int,int,int,int,int);
extern void nst_diamondfi(int,int,int,int);
extern void nst_draw_square(float,float,int);
extern void nst_draw_diamond(float,float,int);
extern void nst_draw_triangle(float,float,int);
extern void nst_draw_cross(float,float,int);
extern void nst_draw_plus(float,float,int);
extern void nst_rectwrite(int,int,int,int,unsigned short int*);
extern void nst_Xrectwrite(int,int,int,int,int,
                           int,int,int,int,int,void*);
extern void nst_XRGBrectwrite(int,int,int,int,int,int,int,int,int, 
                              int,void*,void*,void*);
extern void nst_lrectwrite(int,int,int,int,unsigned long int*);
extern int nst_true_rgb_mode_of(int iNstWindowId);
extern int nst_rgb_3d_mode_of(int iNstWindowId);
extern int nst_read_rectangle(Display*,Window,int,int,int,
                              int, int, int, int, int, void*);
extern int nst_rectread(int,int,int,int,unsigned short int*);
extern int nst_lrectread(int,int,int,int,unsigned long int*);
extern void nst_font(int);
extern int  nst_setfont(int);
extern int  get_font_size(int*,int*,int);
extern int nst_strwidth(char*);
extern void nst_charstr(char*);
#ifdef OGL
extern int nst_readpixels(int,int,int,int,char,void*,void*,void*);
#endif

extern void nst_no_clipping(void);
extern void nst_clip_rectangle(float,float,float,float);

extern void nst_SetXRootWindow(Window pRoot);

#endif
