/**********************************************************************

  Example of wrapping a C++ class for #import by a 
  prog_unit:

  The example class is very simple and has only three exported
  variables (a float and int parameter and a dynamically resizeable 
  array) and a single method (which does the resizeing of the array):

  For an instance w:

  w.change(int iNewDim)  redimension array to new dimension iNewDim
  w.pfFloat              the float array 
  w.fX                   the float variable
  w.iX                   the int variable
 
***********************************************************************/

extern "C" {  // to allow proper linking with C

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/************************************************************************
 definition of a simple example C++ class: 
 (this class will be wrapped below) 
*************************************************************************/

class foo {

  public:

  int iFloatDim;
  int iDynFloatDim;

  // these members will be made visible for the prog_unit 
  float fX, *pfFloat, *pfDynFloat;  
  int iX;

  // the only method (will be made visible for the prog unit): 
  void change(int iFloats) {
     int i;
     if (iFloats >= 0) pfFloat = (float*) realloc(pfFloat,iFloats*sizeof(float));
     for (i=iFloatDim; i<iFloats; i++) pfFloat[i] = 0;
     iFloatDim = iFloats;
  }

  // constructor and destructor: 
  foo(int iFloats = 10 /* default size */) {
     int i;
     pfFloat = (float*) malloc(iFloats*sizeof(float));
     for (i=0; i<iFloats; i++) pfFloat[i] = 0;
     fX = 0; iX = 0;
     iFloatDim = iFloats;

     iDynFloatDim = 2;
     pfDynFloat = (float*) malloc(iDynFloatDim*sizeof(float));
     for (i=0; i<iDynFloatDim; i++) pfDynFloat[i] = 3.14;
  }

  ~foo() { free(pfFloat); }
};

/*************************************************************************
  here comes the wrapper that makes this class visible for the prog unit
  (everything below could work without explicit access to the 
   definition of the wrapped C++ class): 
**************************************************************************/

char *foo_c[] = {
    "L_:",             // prefix against name collision  
    "foo(int)",        // class constructor 
    "~foo()",          // class destructor (mapped to "FREE_foo()") 
    "float fX",        // the float variable
    "int iX",          // an int variable 
    "float* pfFloat",  // the float array 
    "float* pfDyn",    // the dyn float array 
    "change(int)",     // the exported method 
    "COPY_foo()",      // copy constructor to allow INP/OUT & assignments 
    "UPDATE_foo()",    // notifier for dynamic array resize 
    "DESCRIBE_foo()",  // makes description string 
    NULL};

char **CLASSES[] = {foo_c,NULL};

/************************************************************************
  the following function pointer will be set by the prog unit to 
  a suitable notifier function that must be called by any wrapper
  function that makes an address or size change to a class variable
  that is made visible for the prog unit (this definition may be
  omitted, if there are no such variables):
************************************************************************/

void (*L_UPDATE_foo)(void) = NULL;  // to enable the notifier 

/*************************************************************************
  now the definitions of the wrapper interface functions:
  essentially, each function wrapper just converts the parameter list
  of the original class function to the (int*,void**) parameter
  passing of the prog unit loadlib functions:
*************************************************************************/

/*************************************************************************
  wrapper for class constructor:  
*************************************************************************/

class foo *L_foo(int *piDim, void **ppvArg) {

   class foo *pData;
   int *piArg0 = (int*) (ppvArg[0]); // cast adr of 1st arg to expected type
   pData = new foo(*piArg0);
   return pData;
}

/*************************************************************************
  all non constructor-wrappers have the class instance adr as an
  additional last argument parameter:
*************************************************************************/

/*************************************************************************
  wrapper for class destructor:
*************************************************************************/

void L_FREE_foo(class foo *pData) {
   // implementation must permit NULL argument! 
   if (pData) delete pData;
}

/*************************************************************************
  wrapper for class method  change(int): must call function & notifier
*************************************************************************/

float L_change(int *piDim, void **ppvArg, class foo *pData) {
   int *piArg0 = (int*) ppvArg[0];
   pData->change(*piArg0);
   L_UPDATE_foo();  
   /* the prog_unit has provided in L_UPDATE_foo a suitable notifier
      function that must be called whenever a dynamic element variable
      has changed its address or dimension, which is the case here. 
   */
   return 0; 
}

/*************************************************************************
  wrappers for class variables: these must pass back address & dim:
*************************************************************************/

/*************************************************************************
  wrapper for class element variables fX and iX:   must return adress 
*************************************************************************/

float *L_fX(int *piDim, class foo *pData) {  return &(pData->fX); }
int   *L_iX(int *piDim, class foo *pData) {  return &(pData->iX); }

/*************************************************************************
  wrapper for class array element var pfFloat: must return address & dim
*************************************************************************/

float *L_pfFloat(int *piDim, class foo *pData) {  
   *piDim = pData->iFloatDim;  
   return pData->pfFloat;
}

/*************************************************************************
  wrapper for class dynamic array element pfDynFloat
*************************************************************************/

float** L_DYN_pfDyn(int **ppiDim, class foo *pData) {  
   *ppiDim = &pData->iDynFloatDim;  
   return &pData->pfDynFloat;
}

/*************************************************************************

   THIS IS THE BARE MINIMUM. OPTIONAL CONVENIENCES BELOW:

   when more functions are defined, class instances can 
   - take part in load/save operations (requires writing of
     LOAD_foo/SAVE_foo function, not illustrated),
   - be used as INP/OUT pins (requires COPY method shown below)
   - be inspected in INP/OUT pins via popup description text (requires
     writing of DESCRIBE_foo function illustrated below).
 
*************************************************************************/

/*************************************************************************
  copy function to allow use of class as INP/OUT instance:
  (here, the arguments are always two class handles; no
   argument list specification is expected in the interface spec)
*************************************************************************/

class foo *L_COPY_foo(class foo *pTo, class foo *pFrom) {

// important: must cope with all combinations of NULL args! 

   if (!pTo) {
      if (!pFrom) return new foo();  // default instance 
      pTo = new foo(pFrom->iFloatDim);
      memcpy(pTo->pfFloat, pFrom->pfFloat, pFrom->iFloatDim*sizeof(float));
   } else {
      // re-use pTo to the extent possible: 
      if (!pFrom) {
         pTo->change(10); 
         return pTo;
      } else {
         pTo->change(pFrom->iFloatDim);
         memmove(pTo->pfFloat,pFrom->pfFloat, pFrom->iFloatDim*sizeof(float));
      }
   }    
   return pTo;
}

/*************************************************************************
  function to provide info string when instance is in INP/OUT pin:
*************************************************************************/

char *L_DESCRIBE_foo(class foo *pData) {
   static char acDescription[500];
   int i;
   // assemble a description of the data in the class instance: 
   sprintf(acDescription," = %p\n"
           " iX        = %d\n"
           " fX        = %g\n"
           " iFloatDim = %d\n",
           pData,pData->iX,pData->fX,pData->iFloatDim);
   for (i=0; i<5 && i<pData->iFloatDim; i++) {
      sprintf(acDescription+strlen(acDescription)," value[%d]  = %g\n",i,pData->pfFloat[i]);
   }
   if (i<pData->iFloatDim) {
      sprintf(acDescription+strlen(acDescription)," ... \n");
      i = pData->iFloatDim-1;  // to print last element 
      sprintf(acDescription+strlen(acDescription)," value[%d]  = %g\n",i,pData->pfFloat[i]);
   } 
   return acDescription;
}

} /* extern "C" */
