/******************************************************************

  Example program illustrating use of libnstld - routines
  to instantiate *.NST file as NST circuit within C-program.

  For more documentation, see NST_HOME/man/pdf/nstloader.pdf
  and man nst_call

*******************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include "nst.h"
#include "nstld.h"

int main(int iArg, char *apcArg[]) {

    unitptr uCircuit, uPCA; 
    char    *pcNstFile = NULL;
    int     i, iDimVecs;
    float   *pfInp;
    float   *pfOut;
    float   afEigVals[3];

    if (iArg > 1) pcNstFile = apcArg[1];
    else pcNstFile = strdup("PCA.NST.gz"); /* not freed */
                     
    // initialize NST and load some folders
    init_nst(); 
    nst_init_folders();
    nst_load_folders_from_file("neofolders-loader",".");

    // Now, load the specified circuit file.
    // 
    // If the file specifies an "open" circuit (Save As "Entire Circuit")
    // uCircuit points to the host unit!
    //
    // If the file specifies an "encapsulated" circuit (Save As "Diplayed Subcircuit")
    // uCircuit points to the newly instantiaded unit!
    //
    uCircuit = nst_load_circuit_from_file(pcNstFile,NULL,ROOT);

    if (!uCircuit) {
        fprintf(stderr, "failed to load circuit!\n");
        exit(-1);
    }

    // In this example, the PCA.NST file was saved as the "entire circuit",
    // hence it specifies an "open" circuit. Since we want transfer data
    // to and from the "pca" unit, we must get the handle to this unit:
    //
    if (!(uPCA = find_unit_by_name("pca", uCircuit, 0))) {
        fprintf(stderr, "could not find unit with name \"pca\"\n");
        exit(-1);
    }

    // Allocate memory for input data
    pfInp = calloc(3, sizeof(float));

    // Set random seed
    srand48(clock());

    // Choose variances along xyz directions
    for (i=0; i < 3; i++) {
        pfInp[i] = drand48();
        printf("<<< Variance %d = %f\n", i, pfInp[i]);
    } 

    // If pfOut=NULL, nst_call() will allocate sufficient memory,
    // otherwise the data will be copied to the memory pfOut points to
    // (compare with manual page of nst_call())
    pfOut = NULL;

    printf("+++ nstcall now calling unit %p\n", uPCA);
    printf("+++ PRESS RIGHT MOUSE BUTTON IN WINDOW TO RETURN...\n");

    // Execute the "pca" unit and transfer the input and output data
    nst_call(uPCA, NULL, 
             NST_INP, 3, &pfInp,
             NST_OUT, &iDimVecs, &pfOut,
             NST_OUT, 3, &afEigVals,
             NST_END);

    printf("+++ nst_call() returned\n");
    printf("+++ Output Pin 0 dimension %d\n", iDimVecs);

    // Print the output data to stdout
    for (i=0; i < 3; i++) {
        printf(">>> Variance PCA %d = %f", i, sqrt(afEigVals[i]));
        printf("    (%f,%f,%f)\n", pfOut[i*3+0], pfOut[i*3+1], pfOut[i*3+2]);
    }

    return 0;
}


